# This is a -*- Tcl -*- library script and belongs to the
# `Jumble Library for Tcl and Friends'.
#
# debug.tcl - debugging functions
#
# Copyright (C) 1996 Stefan Hornburg
#
# Author: Stefan Hornburg <racke@gundel.han.de>
# Maintainer: Stefan Hornburg <racke@gundel.han.de>
# Version: 0.0
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2, or (at your option) any
# later version.
# 
# This file is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this file; see the file COPYING.  If not, write to the Free
# Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

# ===============
# initializations
# ===============

# ---------------------------------
# setup debug flags for this module
# ---------------------------------

set juf_dbg_flaghash(juf_flags_dbg_root) {
	juf_flags_dbg_warn juf_flags_dbg_flags juf_flags_dbg_print}
set juf_dbg_flaghash(juf_flags_dbg_warn) {}
set juf_dbg_flaghash(juf_flags_dbg_flags) {}
set juf_dbg_flaghash(juf_flags_dbg_print) {}

# -------------------
# disable debug flags
# -------------------

set juf_dbg_statehash(juf_flags_dbg_root) 0 
set juf_dbg_statehash(juf_flags_dbg_warn) 0
set juf_dbg_statehash(juf_flags_dbg_flags) 0
set juf_dbg_statehash(juf_flags_dbg_print) 0

# ------------------------------------------
# setup output channel (default is `stdout')
# ------------------------------------------

set juf_dbg_targets stdout

# ==================
# `debug' procedures
# ==================

# ----------------------------------
# juf_dbg_isflag - validate flag
#
# in:	flag 	flag name
# out:  1 if successful, 0 otherwise
# ----------------------------------

proc juf_dbg_isflag {flag} {
	global juf_dbg_statehash
	juf_dbg_out juf_flags_dbg_flags "Validate flag $flag"
	if [info exists juf_dbg_statehash($flag)] {
		juf_dbg_out juf_flags_dbg_flags\
				"Flag value: $juf_dbg_statehash($flag)"
		return 1
	}
	return 0
}

# ------------------------------------------------------------
# juf_dbg_out - prints debug message if given flag is enabled
#
# in:	flag	Flag for debug message
#		args	Debug message
# ------------------------------------------------------------

proc juf_dbg_out {flag args} {
	global juf_dbg_statehash juf_dbg_targets

	if {[string compare $flag juf_flags_dbg_print] != 0} {
		juf_dbg_out juf_flags_dbg_print "debug output requested for flag $flag and message \"$args\""
	}
	if [info exists juf_dbg_statehash($flag)] {
		if $juf_dbg_statehash($flag) {
			set buf "DEBUG: "
			append buf [join $args ": "]
			foreach target $juf_dbg_targets { puts $target $buf }
		}
	} else {
		juf_dbg_out juf_flags_dbg_warn "warning" $flag "Message depends on an unknown flag"
	}
}		                       

# ------------------------------------------------------------
# juf_dbg_out_eval - evaluates script and prints debug message
#
# in:	flag 	debug flag
#		script	script to be evaluated if `flag' is set
# ------------------------------------------------------------

proc juf_dbg_out_eval {flag script} {
	global juf_dbg_statehash juf_dbg_targets
	
	if {[string compare $flag juf_flags_dbg_print] != 0} {
		juf_dbg_out juf_flags_dbg_print "debug output requested for flag $flag and script \"$script\""
	}
	
	if [info exists juf_dbg_statehash($flag)] {
		if $juf_dbg_statehash($flag) {
			set buf "DEBUG: "
			append buf [uplevel $script]
			foreach target $juf_dbg_targets { puts $target $buf }
		}
	} else {
		juf_dbg_out juf_flags_dbg_warn "warning" $flag "Message depends on an unknown flag"
	}
}

# -----------------------------------------------
# juf_dbg_set_channels - changes output channels
#
# in:	args	list of file identifiers
# -----------------------------------------------

proc juf_dbg_set_channels { args } {
	global juf_dbg_targets 
	set juf_dbg_targets $args
}

# ------------------------------------------
# juf_dbg_add_flags - add debug flags
# 
# in:	args	list of flags and flag lists
# ------------------------------------------

proc juf_dbg_add_flags { args } {
	global juf_dbg_flaghash juf_dbg_statehash
	foreach flaglist $args {
		set flagname [lindex $flaglist 0]
		if {[llength $flaglist] == 1} {
			set juf_dbg_flaghash($flaglist) {}
			juf_dbg_out juf_flags_dbg_flags "debug flag $flaglist created"
		} else {
			set juf_dbg_flaghash([lindex $flaglist 0]) [lrange $flaglist 1 end]
			juf_dbg_out juf_flags_dbg_flags "debug flag [lindex $flaglist 0] created"			
			# Create subflags
			# ===============

			foreach subflag [lrange $flaglist 1 end] {
				juf_dbg_add_flags $subflag
			}
		}
		set juf_dbg_statehash([lindex $flaglist 0]) 0
	} 
}

# -----------------------------------------------------------
# juf_dbg_set_flags - set all given flags and their subflags
# -----------------------------------------------------------

proc juf_dbg_set_flags { args } {
	global juf_dbg_flaghash juf_dbg_statehash
	foreach flag $args {
		# check if flag available
		if [info exists juf_dbg_flaghash($flag)] {
			# enable given flag
			set juf_dbg_statehash($flag) 1
			juf_dbg_out juf_flags_dbg_flags "enabled debug flag $flag"
			if [llength $juf_dbg_flaghash($flag)] {
				# check and enable subflags (recursive)
				eval juf_dbg_set_flags $juf_dbg_flaghash($flag)
			}
		} else {
			juf_err_fat $flag "Undefined debug flag"
		}
	}
}

# ======================================
# debug flags for other `jultaf' modules
# ======================================

# ---------
# `getopts'
# ---------

juf_dbg_add_flags {juf_flags_getopts_arg juf_flags_getopts_arg_before \
		juf_flags_getopts_names juf_flags_getopts_arg_after}






