# This is a -*- Tcl -*- library script and belongs to the
# `Jumble Library for Tcl and Friends'.
#
# error.tcl --- error handling module
#
# Copyright (C) 1996 Stefan Hornburg
#
# Author: Stefan Hornburg <racke@gundel.han.de>
# Maintainer: Stefan Hornburg <racke@gundel.han.de>
# Version: 0.0
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2, or (at your option) any
# later version.
# 
# This file is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this file; see the file COPYING.  If not, write to the Free
# Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

proc juf_err_report {channel args} {

	# Prepend script only if not running interactively
	# ================================================

	if {[info script] != ""} {
		set buf [file rootname [file tail [info script]]]
		if {$args != ""} {
			append buf ": "
		}
	}
	append buf [join $args ": "]

	if {$channel == ""} {
		error $buf
	}
	
	puts $channel $buf
}

# ------------------------------------------------
# juf_err_tclmsg - evaluates `errorCode' variable
# ------------------------------------------------

proc juf_err_tclmsg {} {
	global errorCode

	switch [lindex $errorCode 0] {
		ARITH -
		POSIX {return [lindex $errorCode 2]}
	}

	return "unknown error"
}

# --------------------------------------------------------------------------
# juf_err_warn - prints error message on `stderr' with "warning" prepended
# --------------------------------------------------------------------------

proc juf_err_warn {args} {
	eval juf_err_report stderr "warning" [lrange $args 0 end]
}

# -----------------------------------------------
# juf_err_err - prints error message on `stderr'
# -----------------------------------------------

proc juf_err_err {args} {
	eval juf_err_report stderr [lrange $args 0 end]
}

# ----------------------------------------------------------------------
# juf_err_fat - prints error message on `stderr' and terminates process
# ----------------------------------------------------------------------

proc juf_err_fat {args} {
	eval juf_err_report stderr [lrange $args 0 end]
	exit 1
}


